#!/bin/bash

STREAM_LOG_FILE=stream.log
DATE=$(date +%Y%m%d_%H%M)
RESULT_LOG_FILE='result/result_'$DATE'.log'

WAIT_FOR_LAUNCH=10
WAIT_FOR_LONG_LAUNCH=30
WAIT_FOR_KILL=5

PKG_IDS=()
APP_IDS=()

LONG_LAUNCHING_APP=(
    "org.tizen.example.SNSUI.Tizen",
)

initialize ()
{
    echo ""
    echo "[>] Initialize for Performance Test"
    if [ $(sdb get-state 2>/dev/null | grep -c "device") -eq 0 ];
    then
        echo ""
        echo "[!] device is not connected - cannot execute"
        echo ""
        exit 0
    fi

    if [ $(dpkg-query -W -f='${Status}' inotify-tools 2>/dev/null | grep -c "ok installed") -eq 0 ];
    then
        echo ""
        echo "[!] inotify-tools package should install"
        echo "[!] starting install inotify-tools .. "
        sudo apt-get install inotify-tools
        if [ $(dpkg-query -W -f='${Status}' inotify-tools 2>/dev/null | grep -c "ok installed") -eq 0 ];
        then
            echo ""
            echo "[!] install inotify-tools fail - cannot execute"
            echo ""
            exit 0
        fi
        echo 32768 | sudo tee /proc/sys/fs/inotify/max_user_watches
        echo fs.inotify.max_user_watches=32768 | sudo tee -a /etc/sysctl.conf
        sudo sysctl -p
    fi
    sdb root on
    sdb shell "devicectl display stop">/dev/null 2>&1
    mkdir result>/dev/null 2>&1
    rm $STREAM_LOG_FILE>/dev/null 2>&1
    touch $STREAM_LOG_FILE
}

install_tpk ()
{
#install tpk packages
    echo "[>] Installing package in tpk directory"
    TPKS=($(ls tpk | grep .tpk))


    for item in ${TPKS[*]}
    do
        INSTALL_MSG=$(sdb install tpk/$item | grep start)
        INSTALL_MSG=$(echo $INSTALL_MSG | sed "s/\[/ /g")
        INSTALL_MSG=$(echo $INSTALL_MSG | sed "s/\]/ /g")
        PKG_IDS+=($(echo $INSTALL_MSG | awk '{print $7}' | tr -d '\r'))
        echo " [>] ($(echo $INSTALL_MSG | awk '{print $7}')) installs complete"
    done
}

get_current_tpk_apps ()
{
    echo "[>] Get application list in device"
#    PKG_IDS+=$(
#        sdb shell "su - owner -c 'pkgcmd -l | grep tpk'" |  while read line
#        do
#            APP_LIST_ENTITY=$line
#            APP_LIST_ENTITY=$(echo $APP_LIST_ENTITY | sed "s/\[/ /g")
#            APP_LIST_ENTITY=$(echo $APP_LIST_ENTITY | sed "s/\]/ /g")
#            APP_OWNER=($(echo $APP_LIST_ENTITY | awk '{print $1}'))
#            if [[ $APP_OWNER == 'user' ]]
#            then
#                echo $APP_LIST_ENTITY | awk '{print $6}'
#            fi
#        done | sort | tr -d '\r'
#    )
#In 3.0 mobile / wearable, appfw install all of application to global application
    PKG_IDS+=$(
        sdb shell "su - owner -c 'ls -al /opt/usr/globalapps/ | grep tizenglobalapp'" |  while read line
        do
            APP_LIST_ENTITY=$line
            APP_GLOBAL=($(echo $APP_LIST_ENTITY | awk '{print $3}'))
            if [[ $APP_GLOBAL == 'tizenglobalapp' ]]
            then
                echo $APP_LIST_ENTITY | awk '{print $9}'
            fi
        done | sort | tr -d '\r'
    )
}

make_appid_list ()
{
#get app ids
    echo "[>] Get application id that installed"
    for item in ${PKG_IDS[*]}
    do
        APP_LIST_MSG=$(sdb shell "su - owner -c 'pkginfo --pkg $item' | grep mainappid" | tail -1)
        APP_IDS+=($(echo $APP_LIST_MSG | awk '{print $3}' | tr -d '\r'))
    done
}

initialize_first_launch ()
{
    if [[ -z ${APP_IDS[0]} ]]; then
        echo ""
        echo "[!] No tpk Packages for test"
        echo "[!] Copy tpk files in [./tpk] directory"
        echo ""
        exit 0
    fi
    echo "[>] Initial launch an application"
    APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -s ${APP_IDS[0]}'")
    sleep 10
    APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -t ${APP_IDS[0]}'")
    sleep 5
}

execute_time_stamp_auto ()
{
    echo ""
    echo "[>] Start performance test that applciation launching time"
    echo ""
#execute dlogstreamer
    sdb shell "dlogutil -c"
    sdb shell "dlogutil -v time AUL LAUNCH|grep -E 'launch.*app_request_to_launchpad_for_uid.*request.*appid|Launching:done'" >> $STREAM_LOG_FILE &
    DLOG_STREAMER_PID=$!
#execute timestamp
    /bin/bash ./timestamp.sh $STREAM_LOG_FILE $RESULT_LOG_FILE &
    TIMESTAMP_PID=$!
}

execute_time_stamp_auto_memory ()
{
    echo ""
    echo "[>] Start performance test that applciation launching memory"
    echo ""
    sdb shell "dlogutil -c"
    sdb shell "dlogutil -v time AUL LAUNCH|grep -E 'launch.*app_request_to_launchpad_for_uid.*request.*appid|Launching:done'" >> $STREAM_LOG_FILE &
    DLOG_STREAMER_PID=$!
#execute timestamp
    /bin/bash ./timestamp.sh $STREAM_LOG_FILE $RESULT_LOG_FILE &
    TIMESTAMP_PID=$!
#execute memorystamp
    /bin/bash ./memorystamp.sh $STREAM_LOG_FILE $RESULT_LOG_FILE &
    MEMORYSTAMP_PID=$!
}

execute_time_stamp_manual ()
{
#execute dlogstreamer
    echo ""
    echo "[>] Start performance test that applciation launching time"
    echo ""
    rm $STREAM_LOG_FILE
    touch $STREAM_LOG_FILE
    sdb shell "dlogutil -c"
    sdb shell "dlogutil -v time AUL LAUNCH|grep -E 'launch.*app_request_to_launchpad_for_uid.*request.*appid|Launching:done'" >> $STREAM_LOG_FILE &
    DLOG_STREAMER_PID=$!
#execute timestamp
    /bin/bash ./timestamp.sh $STREAM_LOG_FILE $RESULT_LOG_FILE
    TIMESTAMP_PID=$!
    wait $TIMESTAMP_PID
}

execute_time_stamp_manual_trace ()
{
#execute dlogstreamer
    echo ""
    echo "[>] Start performance test that applciation launching time"
    echo ""
    rm $STREAM_LOG_FILE
    touch $STREAM_LOG_FILE
    sdb shell "dlogutil -c"
    sdb shell "dlogutil -v time AUL LAUNCH|grep -E 'launch.*app_request_to_launchpad_for_uid.*request.*appid|Launching:done'" >> $STREAM_LOG_FILE &
    DLOG_STREAMER_PID=$!
#execute timestamp
    /bin/bash ./timestamp.sh $STREAM_LOG_FILE $RESULT_LOG_FILE &
    TIMESTAMP_PID=$!
#execute ttrace
    ~/tizen-sdk/tools/ttrace/ttrace.py -b 20480 -t 10 -o result/trace.html idle app view am &
    TTRACE_PID=$!
    wait $TTRACE_PID
    rm result/*.ftrace
    rm result/*.raw
}

execute_time_stamp_manual_memory ()
{
    echo ""
    echo "[>] Start performance test that applciation launching memory"
    echo ""
    rm $STREAM_LOG_FILE
    touch $STREAM_LOG_FILE
    sdb shell "dlogutil -c"
    sdb shell "dlogutil -v time AUL LAUNCH|grep -E 'launch.*app_request_to_launchpad_for_uid.*request.*appid|Launching:done'" >> $STREAM_LOG_FILE &
    DLOG_STREAMER_PID=$!
#execute timestamp
    /bin/bash ./timestamp.sh $STREAM_LOG_FILE $RESULT_LOG_FILE &
    TIMESTAMP_PID=$!
#execute memorystamp
    /bin/bash ./memorystamp.sh $STREAM_LOG_FILE $RESULT_LOG_FILE &
    MEMORYSTAMP_PID=$!
    wait $MEMORYSTAMP_PID
}

execute_all_app ()
{
#execute each apps
    for item in ${APP_IDS[*]}
    do
        APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -s $item'")
        if [[ "${LONG_LAUNCHING_APP[@]}" =~ "${item}" ]]; then
            sleep $WAIT_FOR_LONG_LAUNCH
        else
            sleep $WAIT_FOR_LAUNCH
        fi
        APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -t $item'")
        sleep $WAIT_FOR_KILL
    done
}

execute_all_app_trace ()
{
#execute each apps
    for item in ${APP_IDS[*]}
    do
        ~/tizen-sdk/tools/ttrace/ttrace.py -b 20480 -t 13 -o result/${item}.html idle app view am & > /dev/null 2>&1
        TTRACE_PID=$!
        sleep 1
        APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -s $item'")
        if [[ "${LONG_LAUNCHING_APP[@]}" =~ "${item}" ]]; then
            sleep $WAIT_FOR_LONG_LAUNCH
        else
        sleep $WAIT_FOR_LAUNCH
        fi
        APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -t $item'")
        sleep $WAIT_FOR_KILL
        sleep 4
    done
    rm result/*.ftrace
    rm result/*.raw
}

execute_all_app_memory ()
{
    sleep $WAIT_FOR_LAUNCH
#execute each apps
    for item in ${APP_IDS[*]}
    do
        if [[ "${LONG_LAUNCHING_APP[@]}" =~ "${item}" ]]; then
            echo ""
        else
            APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -s $item'")
            sleep $WAIT_FOR_LONG_LAUNCH
            #APP_LIST_MSG=$(sdb shell "su - owner -c 'app_launcher -t $item'")
            sleep $WAIT_FOR_KILL
        fi
        sleep $WAIT_FOR_LONG_LAUNCH
    done
}

finalize ()
{
    echo ""
    echo "[>] Result"
    echo ""
    cat $RESULT_LOG_FILE
    echo ""
    echo "[>] Result file : [ $RESULT_LOG_FILE ]"
}

destory ()
{
    echo ""
    kill -9 $DLOG_STREAMER_PID>/dev/null 2>&1
    kill -9 $TIMESTAMP_PID>/dev/null 2>&1
    kill -9 $TTRACE_PID>/dev/null 2>&1
    kill -9 $MEMORYSTAMP_PID>/dev/null 2>&1
    rm $STREAM_LOG_FILE>/dev/null 2>&1
    sdb shell "devicectl display start">/dev/null 2>&1
    echo "[>] Finalize for Performance Test"
    echo ""
}

help ()
{
    echo ""
    echo "[!] usage   : <script> [option]"
    echo "    options : -a --auto             full automatic launching time test"
    echo "            : (suboption for auto) -s --skip-install  skip install tpk (execute tpk app in device)"
    echo "            : -m --manual           execute application manually"
    echo "            : -am --auto-memory     full automatic launching memory test"
    echo "            : -mm --manual-memory   execute application manually with memory test"
    echo "            : -at --auto-ttrace     full automatic launching time test with ttrace"
    echo "            : -mt --manual-ttrace   execute application manually with ttrace"
    echo "example : ./performance_test.sh --auto"
    echo ""
}


trap "destory" 0

if [[ $1 == '-a' ]] || [[ $1 == "--auto" ]]
then
    echo ""
    echo "[>] Start with AUTO mode"
    initialize
    if [[ $2 == "-s" ]] || [[ $2 == "--skip-install" ]]
    then
        get_current_tpk_apps
    else
        install_tpk
    fi
    make_appid_list
    initialize_first_launch
    execute_time_stamp_auto
    execute_all_app
    finalize
elif [[ $1 == "-m" ]] || [[ $1 == "--manual" ]]
then
    echo ""
    echo "[>] Start with MANUAL mode"
    echo "[>] execute installed application in your device"
    initialize
    execute_time_stamp_manual
    finalize
elif [[ $1 == "-am" ]] || [[ $1 == "--auto-memory" ]]
then
    echo ""
    echo "[>] Launching Memory Profiling with AUTO mode"
    initialize
    if [[ $2 == "-s" ]] || [[ $2 == "--skip-install" ]]
    then
        get_current_tpk_apps
    else
        install_tpk
    fi
    make_appid_list
    initialize_first_launch
    execute_time_stamp_auto_memory
    execute_all_app_memory
    finalize
elif [[ $1 == "-mm" ]] || [[ $1 == "--manual-memory" ]]
then
    echo ""
    echo "[>] Launching Memory Profiling with MANUAL mode"
    initialize
    execute_time_stamp_manual_memory
    finalize
elif [[ $1 == "-at" ]] || [[ $1 == "--auto-ttrace" ]]
then
    echo ""
    echo "[>] Start with T-trace Auto mode"
    initialize
    if [[ $2 == "-s" ]] || [[ $2 == "--skip-install" ]]
    then
        get_current_tpk_apps
    else
        install_tpk
    fi
    make_appid_list
    initialize_first_launch
    execute_time_stamp_auto
    execute_all_app_trace
    finalize
elif [[ $1 == "-mt" ]] || [[ $1 == "--manual-ttrace" ]]
then
    echo ""
    echo "[>] Start with T-trace Manual mode"
    echo "[>] execute installed application in your device"
    initialize
    execute_time_stamp_manual_trace
    finalize
else
    help
fi
